const taapi = require("taapi");

const axios = require("axios");

const config = require("./config/config");

const database = require("./utilities/Database");

class TaapiBot {
  constructor() {
    this.name = null;
    this.states = {};
    this.fingerprint = null;
    this.lastCandleTimestamp = null;
    this.taapiSecret = null;
    this.taapiEndpoint = "https://ta.taapi.io";
    this.client = null;
    this.exchangeData = null;
  }

  async init(name) {
    this.name = name;
    await config.validateEnvironment();
    await database.updateStrategy(name, "some property");
  }

  initTaapi(secret, endpoint) {
    this.taapiSecret = secret;
    this.client = taapi.client(secret);
    this.exchangeData = taapi.exchangeData();

    if (endpoint) {
      this.taapiEndpoint = endpoint;
      this.client.setTaapiApiEndpoint(endpoint);
    }
  }

  initTaapiCache(secret, endpoint) {
    this.client.setDataProviderTaapiCache(secret, endpoint);
    this.exchangeData.setDataProviderTaapiCache(secret, endpoint);
  }

  getLastCandle(candles) {
    return candles[candles.length - 1];
  }

  getPreviousCandle(candles) {
    return candles[candles.length - 2];
  }

  async getCandles(source, symbol, interval, backtrack, candleCount) {
    let fixedCandles = [];
    await this.exchangeData.getCandles(source, symbol, interval, backtrack, candleCount).then(async candles => {
      for (let c in candles) {
        const candle = candles[c];
        fixedCandles.push({
          timestamp: parseInt(candle.timestamp),
          open: parseFloat(candle.open),
          close: parseFloat(candle.close),
          low: parseFloat(candle.low),
          high: parseFloat(candle.high),
          volume: parseFloat(candle.volume)
        });
      }
    }).catch(function (error) {
      console.error(error);
    });
    return fixedCandles;
  }

  async tick(source, symbol, interval, backtrack, candleCount) {
    const self = this;
    const candles = await this.getCandles(source, symbol, interval, backtrack, candleCount);
    await this.run(source, symbol, interval, candles);
  }

  async run(source, symbol, interval, candles) {
    const self = this;

    try {
      this.fingerprint = `${source}_${symbol}_${interval}`;
      let lastCandle = candles[candles.length - 1];
      this.lastCandleTimestamp = lastCandle.timestamp / 1000;
      await database.getState(this.name, this.fingerprint).then(async state => {
        if (!state) {
          state = await database.updateState(self.name, "start", this.fingerprint, this.lastCandleTimestamp);
        }

        if (!state.data) {
          state.data = {};
        }

        self.states[state.stateName](self.client, candles, state);
      }).catch(error => {
        console.error(error);
      });
    } catch (error) {
      console.log("Something bad happened with this symbol!"); //console.error(error);
    }
  }

  async backTest(source, symbol, interval) {
    let self = this;
    let fingerprint = `${source}_${symbol}_${interval}`;
    await database.deleteState(this.name, fingerprint);
    await database.deleteStateChanges(this.name, fingerprint);
    let candles = await this.getCandles(source, symbol, interval, 0, -1);
    let batchSize = 300;

    for (let i = batchSize; i < candles.length; i++) {
      let tickCandles = candles.slice(i - batchSize, i); //let candle = candles[i];

      let lastCandle = tickCandles[tickCandles.length - 1]; //console.log(`Tick: ${new Date(lastCandle.timestamp)}`);

      await self.run(source, symbol, interval, tickCandles);
      await new Promise(resolve => setTimeout(resolve, 1000 / 2));
    }
  }

  async changeState(stateName, data) {
    if (!data) {
      data = {};
    }

    await database.updateState(this.name, stateName, this.fingerprint, this.lastCandleTimestamp, data);
  }

  async getIndicator(indicator, candles, params) {
    if (!params) {
      params = {};
    }

    params.secret = this.taapiSecret;
    params.candles = candles;
    let result = null;
    await axios.post(`${this.taapiEndpoint}/${indicator}`, params).then(function (response) {
      result = response.data;
    }).catch(function (error) {
      console.log(error.response.data);
    });
    return result;
  }

  async getIndicators(indicators, candles) {
    let result = [];
    this.client.initRawBulkQueries();
    this.client.setRawCandles(candles);

    for (let indicatorKey in indicators) {
      this.client.addRawBulkQuery(indicators[indicatorKey]);
    }

    await this.client.executeRawBulkQueries().then(response => {
      result = response;
    }).catch(error => {
      console.log(error);
    });
    return result;
  }

  addStartState(resolver) {
    this.states.start = resolver;
  }

  addState(name, resolver) {
    this.states[name] = resolver;
  }

  async getCachedSymbols() {
    let result = null;
    await axios.get(`http://localhost:6043/tickers?secret=${config.auth.secret}&provider=alphavantage&interval=1d`).then(function (response) {
      result = response.data.data;
    }).catch(function (error) {
      console.log(error.response.data);
    });
    return result;
  }

  async getCachedCandles(symbol) {
    let result = null;
    await axios.get(`http://localhost:6043/candles?secret=${config.auth.secret}&provider=alphavantage&source=equity&symbol=${symbol}&interval=1d&limit=300`).then(function (response) {
      result = response.data.data;
    }).catch(function (error) {
      console.log(error.response.data);
    });
    return result;
  }

  normalizeValue(value, min, max) {
    return (value - min) / (max - min);
  }

  round(value, accuracy) {
    accuracy = Math.pow(10, accuracy || 0);
    return Math.round(value * accuracy) / accuracy;
  }

}

module.exports = new TaapiBot();