const taapi = require("taapi");

const axios = require("axios");

const config = require("./config/config");

const database = require("./utilities/Database");

class TaapiBotDirect {
  constructor() {
    this.name = null;
    this.states = {};
    this.fingerprint = null;
    this.lastTimestamp = null;
    this.taapiSecret = null;
    this.taapiEndpoint = "https://ta.taapi.io";
    this.client = null;
    this.exchangeData = null;
    this.type = null;
    this.interval = null;
    this.symbol = null;
    this.exchange = null;
    this.notificationsEnabled = true;
  }

  async init(name) {
    this.name = name;
    await config.validateEnvironment();
    await database.updateStrategy(name, "some property");
  }

  initTaapi(secret, endpoint) {
    this.taapiSecret = secret;
    this.client = taapi.client(secret);
    this.exchangeData = taapi.exchangeData();

    if (endpoint) {
      this.taapiEndpoint = endpoint;
      this.client.setTaapiApiEndpoint(endpoint);
    }
  }

  setNotificationsEnabled(notificationsEnabled) {
    this.notificationsEnabled = notificationsEnabled;
  }

  setType(type) {
    this.type = type;
  }

  getInterval() {
    return this.interval;
  }

  setInterval(interval) {
    this.interval = interval;
  }

  setExchange(exchange) {
    this.exchange = exchange;
  }

  async tick(name, symbol) {
    this.symbol = symbol;
    this.fingerprint = `${name}_${symbol}`;
    await this.run();
  }

  async getState() {
    return await database.getState(this.name, this.fingerprint);
  }

  async run() {
    const self = this;

    try {
      this.lastTimestamp = new Date().getTime() / 1000;
      await database.getState(this.name, this.fingerprint).then(async state => {
        if (!state) {
          state = await database.updateState(self.name, "start", this.fingerprint, this.lastTimestamp);
        } //console.log(`Running: ${this.fingerprint}:${state.stateName}`);


        if (!state.data) {
          state.data = {};
        }

        self.states[state.stateName](self.client, state);
      }).catch(error => {
        console.error(error);
      });
    } catch (error) {
      console.log("Something bad happened with this symbol!"); //console.error(error);
    }
  }

  postSlackMessage(message, fields) {
    database.getState(this.name, this.fingerprint).then(async state => {
      //let postMessage = `${this.fingerprint}:${state.stateName}:${this.interval}: ${message}`;
      if (fields) {
        fields["Strategy"] = this.name;
        fields["Instance"] = this.fingerprint;
        fields["Current state"] = state.stateName;
        fields["Current timeframe"] = this.interval;
      } else {
        fields = {
          "Strategy": this.name,
          "Instance": this.fingerprint,
          "Current state": state.stateName,
          "Current timeframe": this.interval
        };
      }

      if (this.notificationsEnabled) {
        this.client.postSlackhMessage(message, fields);
      } else {
        console.log(`SLACK MESSAGE: ${message}`, fields);
      }
    });
  }

  printMessage(message) {
    if (config.server.debugMode) {
      database.getState(this.name, this.fingerprint).then(async state => {
        console.log(`${this.fingerprint}:${state.stateName}:${this.interval}: ${message}`);
      });
    }
  }

  async changeState(stateName, data) {
    if (!data) {
      data = {};
    }

    await database.updateState(this.name, stateName, this.fingerprint, this.lastTimestamp, data);
  }

  async getIndicator(indicator, interval, params, symbol) {
    if (!symbol) {
      symbol = this.symbol;
    }

    if (!params) {
      params = {};
    }

    params.secret = this.taapiSecret;
    params.type = this.type;
    params.exchange = this.exchange, params.symbol = symbol;
    params.interval = interval;
    let result = null;
    await axios.get(`${this.taapiEndpoint}/${indicator}`, {
      params: params
    }).then(function (response) {
      result = response.data;
    }).catch(function (error) {
      console.log(error.response.data);
    });
    return result;
  }

  async getIndicators(indicators, interval, symbol) {
    if (!symbol) {
      symbol = this.symbol;
    }

    if (!interval) {
      interval = this.interval;
    }

    let query = {
      "secret": this.taapiSecret,
      "construct": {
        "symbol": symbol,
        "type": this.type,
        "exchange": this.exchange,
        "interval": interval,
        "indicators": indicators
      }
    };
    let result = null;
    await axios.post(`${this.taapiEndpoint}/bulk`, query).then(function (response) {
      result = response.data.data;
    }).catch(function (error) {
      console.log(error.response.data);
    });
    return result;
  }

  findBulkResult(id, indicators) {
    let result = indicators.find(indicator => {
      return indicator.id === id;
    });

    if (result.errors > 0) {
      console.error(result.errors);
    } else {
      return result.result;
    }

    return null;
  }

  async sleep(time) {
    await new Promise(resolve => setTimeout(resolve, time));
  }

  addStartState(resolver) {
    this.states.start = resolver;
  }

  addState(name, resolver) {
    this.states[name] = resolver;
  }

  normalizeValue(value, min, max) {
    return (value - min) / (max - min);
  }

  round(value, accuracy) {
    accuracy = Math.pow(10, accuracy || 0);
    return Math.round(value * accuracy) / accuracy;
  }

  getClient() {
    return this.client;
  }

}

module.exports = new TaapiBotDirect();